@@ -6,11 +6,11 @@ App::Docker::Client - Simple and plain Docker client!
 
 =head1 VERSION
 
-Version 0.010100
+Version 0.010200
 
 =cut
 
-our $VERSION = '0.010100';
+our $VERSION = '0.010200';
 
 use 5.16.0;
 use strict;
@@ -36,11 +36,42 @@ sub new {
 
 =head1 SUBROUTINES/METHODS
 
+=head2 with_valid_code
+
 =cut
 
+sub with_valid_code {
+    my ( $self, $code ) = @_;
+    return if $code !~ m/^(\d{3})$/;
+    $self->{_valid_codes}->{$code} = 1;
+    return $self;
+}
+
+=head2 without_valid_code
+
+=cut
+
+sub without_valid_code {
+    my ( $self, $code ) = @_;
+    return if $code !~ m/^(\d{3})$/;
+    $self->{_valid_codes}->{$code} = 0;
+    return $self;
+}
+
+=head2 show_progress
+
+=cut
+
+sub show_progress {
+    my ( $self, $show_progress ) = @_;
+    $self->{show_progress} = $show_progress ? $show_progress == 0 ? () : 1 : 1;
+    return $self;
+}
+
 =head2 authority
 
 Getter/Setter for internal hash key authority.
+
 =cut
 
 sub authority {
@@ -52,6 +83,7 @@ sub authority {
 =head2 scheme
 
 Getter/Setter for internal hash key scheme.
+
 =cut
 
 sub scheme {
@@ -60,9 +92,22 @@ sub scheme {
     return $_[0]->{scheme};
 }
 
+=head2 ssl_opts
+
+Getter/Setter for internal hash key ssl_opts.
+
+=cut
+
+sub ssl_opts {
+    return $_[0]->{ssl_opts} unless $_[1];
+    $_[0]->{ssl_opts} = $_[1];
+    return $_[0]->{ssl_opts};
+}
+
 =head2 json
 
 Getter/Setter for internal hash key json.
+
 =cut
 
 sub json {
@@ -88,49 +133,53 @@ sub user_agent {
     if ( -S $self->authority() ) {
         LWP::Protocol::implementor( http => 'LWP::Protocol::http::SocketUnixAlt' );
     }
-    $self->{UserAgent} = LWP::UserAgent->new();
+    $self->{UserAgent} = LWP::UserAgent->new(
+        (
+            $self->{show_progress} ? ( show_progress => 1 ) : (),
+            $self->ssl_opts ? ( ssl_opts => $self->ssl_opts ) : (),
+        )
+    );
     return $self->{UserAgent};
 
 }
 
 =head2 get
 
-
-
 =cut
 
 sub get {
-    my ( $self, $path, $options ) = @_;
-    return $self->_to_hashref( $self->user_agent->get( $self->uri( $path, %$options ) ) );
+    my ( $self, $path, $options, $callback ) = @_;
+    return $self->_hande_response( $self->user_agent->get( $self->uri( $path, %$options ) ) );
 }
 
 =head2 delete
 
-
-
 =cut
 
 sub delete {
-    my ( $self, $path, $options ) = @_;
-    return $self->_to_hashref( $self->user_agent->delete( $self->uri( $path, %$options ) ) );
+    my ( $self, $path, $options, $callback ) = @_;
+    return $self->_hande_response( $self->user_agent->delete( $self->uri( $path, %$options ) ) );
 }
 
-
 =head2 request
 
 =cut
 
-sub request { $_[0]->_to_hashref( $_[0]->user_agent->request( $_[1] ) ) }
+sub request {
+    my ( $self, $request, $callback ) = @_;
+    return $self->_hande_response( $self->user_agent->request( $request, $callback ) );
+}
 
 =head2 post
 
 =cut
 
 sub post {
-    my ( $self, $path, $query, $body, $options ) = @_;
-    return $self->request( $self->_http_request( $path, $query ) ) unless $body;
-    return $self->request( $self->_handle_json( $path, $query, $body ) ) unless ($options);
-    return $self->request( $self->_handle_custom( $path, $query, $body, $options ) );
+    my ( $self, $path, $query, $body, $options, $callback ) = @_;
+    return $self->request( $self->_http_request( $path, $query ), $callback ) unless $body;
+    return $self->request( $self->_handle_json( $path, $query, $body ), $callback )
+      unless ($options);
+    return $self->request( $self->_handle_custom( $path, $query, $body, $options ), $callback );
 }
 
 =head2 uri
@@ -155,14 +204,28 @@ sub uri {
     my ( $self, $path, %opts ) = @_;
     require URI;
     my $uri =
-      URI->new( $self->scheme()
-          . $self->{_scheme_postfixes}->{ lc $self->scheme() }
-          . $self->authority() . '/'
-          . $path );
+      URI->new(
+        $self->scheme() . $self->{_scheme_postfixes}->{ lc $self->scheme() } . $self->authority() . '/' . $path );
     $uri->query_form(%opts);
     return $uri;
 }
 
+=head2 to_hashref
+
+Getter/Setter for internal hash key ua.
+
+=cut
+
+sub to_hashref {
+    my ( $self, $content ) = @_;
+    return if !$content;
+    my $data = eval { $self->json->decode($content) };
+    return $@ ? $content : $data;
+    require Carp;
+    Carp::cluck;
+    Carp::croak "JSON ERROR: $@";
+}
+
 {
 
 =head2 post
@@ -206,22 +269,15 @@ create HTTP::Request by uri params
         return HTTP::Request->new( POST => $self->uri( $path, %$query ) );
     }
 
-=head2 _to_hashref
-
-Getter/Setter for internal hash key ua.
+=head2 _hande_response
 
 =cut
 
-    sub _to_hashref {
+    sub _hande_response {
         my ( $self, $response ) = @_;
         $self->_error_code( $response->code, $response->message, $response->content );
         my $content = $response->content();
-        return if !$content;
-        my $data = eval { $self->json->decode($content) };
-        return $@ ? $content : $data;
-        require Carp;
-        Carp::cluck;
-        Carp::croak "JSON ERROR: $@";
+        return $self->to_hashref($content);
     }
 
 =head2 _error_code
@@ -232,7 +288,7 @@ Simple error handler returns undef if everything is ok dies on error.
 
     sub _error_code {
         my ( $self, $code, $message, $content ) = @_;
-        return if $self->{_valid_codes}->{$code};
+        return $code if $self->{_valid_codes}->{$code};
         require Carp;
         Carp::cluck;
         Carp::croak "FAILURE: $code - " . qq~$message\n$content~;
